package com.yihu.base.router.ratelimit;

import com.yihu.base.router.model.Ratelimit;
import com.yihu.base.router.model.RuleRoute;
import com.yihu.base.router.ratelimit.storage.IRateLimitStorage;
import com.yihu.base.router.util.NetworkUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.AntPathMatcher;
import org.springframework.util.StringUtils;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * Created by chenweida on 2018/4/27 0027.
 * IP 规则验证器
 */
public class IPRateLimiter implements IRatelimiter {
    private Logger logger = LoggerFactory.getLogger(IPRateLimiter.class);

    private IRateLimitStorage rateLimitStorage;

    private AntPathMatcher antPathMatcher = new AntPathMatcher();

    @Override
    public Boolean rateLimit(HttpServletRequest request) {
        try {
            //获取请求的的链接
            String path = request.getRequestURI();
            logger.info("path:" + path);
            //获取ip
            String requestIP = NetworkUtil.getIpAddress(request);
            logger.info("ip:" + requestIP);
            //获取数据库规则
            List<Ratelimit> ratelimitList = rateLimitStorage.findAllRatelimit();
            //便利判断
            if (ratelimitList != null && ratelimitList.size() > 0) {
                for (int i = 0; i < ratelimitList.size(); i++) {
                    Ratelimit ratelimit = ratelimitList.get(i);

                    if (StringUtils.isEmpty(ratelimit.getIp())) {
                        //如果数据是空跳过
                        logger.error("ratelimitId:" + ratelimit.getId() + " ip is null");
                        continue;
                    } else {
                        //1.验证当前请求是否符合规则
                        if (!antPathMatcher.match(ratelimit.getPath(), path)) {
                            continue;
                        }
                        //2.判断当前请求的ip是不是可以通过的ip
                        if (requestIP.equals(ratelimit.getIp())) {
                            //3 判断是否有次数限制
                            //默认是0 表示没有限制
                            if (ratelimit.getAccessCount() == 0) {
                                return true;
                            }
                            //如果没有超过次数限制
                            if (!rateLimitStorage.isUpperLimit(ratelimit.getId())) {
                                if (!rateLimitStorage.countDec(ratelimit.getId())) {
                                    return false;
                                }
                                return true;
                            } else {
                                return false;
                            }

                        } else {
                            return false;
                        }
                    }
                }
            }

            return true;
        } catch (Exception e) {
            e.printStackTrace();
            logger.error(e.getMessage());
            return false;
        }
    }

    public IRateLimitStorage getRateLimitStorage() {
        return rateLimitStorage;
    }

    public void setRateLimitStorage(IRateLimitStorage rateLimitStorage) {
        this.rateLimitStorage = rateLimitStorage;
    }
}
